@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
+# Copyright (C) 2012-2014 Peter Hatina <phatina@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,6 +29,8 @@ from LMIDecorators import lmi_return_val_if_fail
 from LMIDecorators import lmi_return_if_fail
 
 from LMIUtil import lmi_cast_to_cim
+from LMIUtil import lmi_get_use_exceptions
+from LMIUtil import lmi_set_use_exceptions
 from LMIUtil import lmi_raise_or_dump_exception
 from LMIUtil import lmi_wrap_cim_instance
 from LMIUtil import lmi_wrap_cim_instance_name
@@ -46,6 +48,7 @@ class LMIClass(LMIWrapperBaseObject):
         # combo __setattr__ and __getattr__
         self.__dict__["_namespace"] = namespace
         self.__dict__["_cim_class"] = None
+        self.__dict__["_cim_class_full_fetch"] = False
         self.__dict__["_cim_classname"] = classname
         super(LMIClass, self).__init__(conn)
 
@@ -55,7 +58,6 @@ class LMIClass(LMIWrapperBaseObject):
         """
         return "%s(classname='%s', ...)" % (self.__class__.__name__, self.classname)
 
-    @lmi_class_fetch_lazy
     def __getattr__(self, name):
         """
         Returns either a class member, or a constant value.
@@ -69,11 +71,13 @@ class LMIClass(LMIWrapperBaseObject):
         if name in self.__dict__:
             return self.__dict__[name]
         if name.endswith("Values"):
+            if not self.is_fetched(True):
+                self.fetch(True)
             property_name = name[:-6]
             return LMIConstantValuesParamProp(self._cim_class.properties[property_name])
         raise AttributeError(name)
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy()
     @lmi_return_val_if_fail(lambda obj: obj._namespace, None)
     def create_instance(self, properties=None, qualifiers=None, property_list=None):
         """
@@ -108,7 +112,7 @@ class LMIClass(LMIWrapperBaseObject):
         return lmi_wrap_cim_instance(self._conn, cim_instance, cim_instance.classname,
             cim_instance.path.namespace)
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy(True)
     @lmi_return_if_fail(lambda obj: obj._namespace)
     def doc(self):
         """
@@ -119,21 +123,47 @@ class LMIClass(LMIWrapperBaseObject):
         """
         LMIXmlFormatter(self._cim_class.tocimxml()).fancy_format(self._conn._client.interactive)
 
-    def fetch(self):
+    def is_fetched(self, full_fetch=False):
+        """
+        Returns True, if :py:class:`CIMClass` has been fetched.
+
+        :param bool full_fetch: defines, if qualifiers are also included
+        """
+        return bool(self._cim_class) and (not full_fetch or self._cim_class_full_fetch)
+
+    def fetch(self, full_fetch=False):
         """
         Manually fetches a wrapped :py:class:`CIMClass` object.
 
+        :param bool full_fetch: True, if :py:class:`CIMClass` should include qualifiers
+            and class origin. Default value is False.
+        :raises: :py:exc:`pywbem.CIMError`, :py:exc:`pywbem.AuthError`
+
         **Usage:** See :ref:`class_fetching_a_class`.
         """
-        if self._cim_class or not self._namespace:
-            # We already have CIMClass, or we do not know the namespace, from which
-            # the class should be fetched.
+        if self.is_fetched(full_fetch) or not self._namespace:
+            # We already have CIMClass (at certain level of detail), or we do
+            # not know the namespace, from which the class should be fetched.
             return
-        (self._cim_class, _, _) = self._conn._client._get_class(self._cim_classname,
-            self._namespace.name, LocalOnly=False)
-        # Store the constant values as a list. This can consume some time, if computed on demand.
-        self._valuemap_properties_list = [k for (k, v) in self._cim_class.properties.iteritems() \
-            if "ValueMap" in v.qualifiers]
+
+        use_exceptions = lmi_get_use_exceptions()
+        lmi_set_use_exceptions(True)
+        try:
+            (self._cim_class, _, _) = self._conn._client._get_class(
+                self._cim_classname,
+                self._namespace.name,
+                full_fetch=full_fetch,
+                LocalOnly=False)
+        except:
+            raise
+        finally:
+            lmi_set_use_exceptions(use_exceptions)
+
+        if full_fetch:
+            self._cim_class_full_fetch = full_fetch
+            # Store the constant values as a list. This can consume some time, if computed on demand.
+            self._valuemap_properties_list = [k for (k, v) in self._cim_class.properties.iteritems() \
+                if "ValueMap" in v.qualifiers and "Values" in v.qualifiers]
 
     # NOTE: usage with Key=something, Value=something is deprecated
     # NOTE: inst_filter is either None or dict
@@ -175,6 +205,9 @@ class LMIClass(LMIWrapperBaseObject):
         for key, value in keybindings.iteritems():
             if isinstance(value, LMIObjectFactory().LMIInstanceName):
                 value = value.wrapped_object
+            elif isinstance(value, str):
+                # Convert strings to unicode
+                value = unicode(value, "utf-8")
             kbs[key] = value
         cim_inst_name = pywbem.CIMInstanceName(self.classname, kbs,
             namespace=self.namespace)
@@ -259,28 +292,28 @@ class LMIClass(LMIWrapperBaseObject):
         inst_name = self.first_instance_name(inst_filter, **kwargs)
         return inst_name.to_instance() if inst_name else None
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy(True)
     @lmi_return_val_if_fail(lambda obj: obj._namespace, [])
     def valuemap_properties(self):
         """
         :returns: list of strings of the constant names
 
-        **Usage:** :ref:`class_valuemap_properties`.
+        **Usage:** :ref:`class_get_valuemap_properties`.
         """
         return self._valuemap_properties_list
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy(True)
     @lmi_return_if_fail(lambda obj: obj._namespace)
     def print_valuemap_properties(self):
         """
         Prints out the list of string of constant names.
 
-        **Usage:** :ref:`class_valuemap_properties`.
+        **Usage:** :ref:`class_get_valuemap_properties`.
         """
         for i in self._valuemap_properties_list:
             sys.stdout.write("%s\n" % i)
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy()
     @lmi_return_val_if_fail(lambda obj: obj._namespace, [])
     def properties(self):
         """
@@ -290,7 +323,7 @@ class LMIClass(LMIWrapperBaseObject):
         """
         return self._cim_class.properties.keys()
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy()
     @lmi_return_if_fail(lambda obj: obj._namespace)
     def print_properties(self):
         """
@@ -301,17 +334,29 @@ class LMIClass(LMIWrapperBaseObject):
         for prop in self._cim_class.properties.keys():
             sys.stdout.write("%s\n" % prop)
 
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy()
     @lmi_return_val_if_fail(lambda obj: obj._namespace, [])
     def methods(self):
         """
         :returns: list of strings of :py:class:`CIMClass` methods.
 
         **Usage:** See :ref:`class_methods`.
+        **Note:** When caching is turned off, this method may consume some time.
         """
-        return self._cim_class.methods.keys()
+        (class_names, _, _) = self._conn._client._get_class_names(self.namespace)
+        methods_lst = self._cim_class.methods.keys()
+        # Iterate through a list of all CIM methods and append any method which
+        # can be called in the synchronous way. One type of such method is the
+        # method, which has its "counterpart" class __MethodParameters_<Method>.
+        # The other one is the method, which defines a Job return paremeter.
+        for (name, method) in self._cim_class.methods.iteritems():
+            if "__MethodParameters_" + name in class_names or \
+                    "Job" in method.parameters:
+                methods_lst.append("Sync" + name)
+        return methods_lst
 
-    @lmi_class_fetch_lazy
+
+    @lmi_class_fetch_lazy()
     @lmi_return_if_fail(lambda obj: obj._namespace)
     def print_methods(self):
         """
@@ -319,7 +364,7 @@ class LMIClass(LMIWrapperBaseObject):
 
         **Usage:** See :ref:`class_methods`.
         """
-        for method in self._cim_class.methods.keys():
+        for method in self.methods():
             sys.stdout.write("%s\n" % method)
 
     @property
@@ -340,7 +385,7 @@ class LMIClass(LMIWrapperBaseObject):
         return self._namespace.name
 
     @property
-    @lmi_class_fetch_lazy
+    @lmi_class_fetch_lazy()
     def wrapped_object(self):
         """
         :returns: wrapped :py:class:`CIMClass` object
