@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
+# Copyright (C) 2012-2014 Peter Hatina <phatina@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,19 +28,16 @@ from LMIExceptions import LMINoPagerError
 
 def _is_executable(fpath):
     """
-    Returns True, if provided path is executable, False otherwise.
-
-    Arguments:
-        fpath -- string containing a path to executable
+    :param string fpath: path of executable
+    :returns: True, if provided path is executable, False otherwise
     """
     return os.path.exists(fpath) and os.access(fpath, os.X_OK)
 
 def _which(program):
     """
-    Returns a full path of a selected binary.
-
-    Arguments:
-        program -- string containing a path to executable
+    :param string program: executable name
+    :returns: full path of a selected executable
+    :rtype: string
     """
     (fpath, fname) = os.path.split(program)
     if fpath:
@@ -55,7 +52,7 @@ def _which(program):
 
 def _get_pager_with_params():
     """
-    Returns a list containing a binary with its CLI arguments.
+    :returns: list containing a executable with its CLI arguments
     """
     if "PAGER" in os.environ:
         path = _which(os.environ["PAGER"])
@@ -74,28 +71,25 @@ def _get_pager_with_params():
 class LMITextFormatter(object):
     """
     Text formatter class. Used when printing a block of text to output stream.
+
+    :param string text: text to be formatted
     """
     def __init__(self, text):
-        """
-        Constructs a LMITextFormatter object.
-        Arguments:
-            text -- text to be formatted.
-        """
         self._text = text
 
     def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout, **kwargs):
         """
         Formats a block of text and prints it to the output stream.
 
-        Arguments:
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
-            kwargs -- dictionary based arguments, supported arguments:
-              separator -- if True, there will be a new line appended after
-                the formatted text
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
+        :param dictionary kwargs: supported keyword arguments
+
+            * **separator** (*bool*) -- if True, there will be a new line appended after
+              the formatted text
         """
         if indent > width:
             return # NOTE: this is wrong!
@@ -120,25 +114,22 @@ class LMIFormatter(object):
     @abc.abstractmethod
     def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout):
         """
-        Abstract method, which ought to be overriden in derived classes.
+        Formats a block of text and prints it to the output stream.
 
-        Arguments:
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
         """
         pass
 
     def fancy_format(self, interactive):
         """
-        Formats a block of text. If the LMIShell is running in interactive
-        mode, pager will be used, otherwise the output will be written
-        to standard output.
+        Formats a block of text. If the LMIShell is running in interactive mode, pager
+        will be used, otherwise the output will be written to standard output.
 
-        Arguments:
-            interactive -- defines, if to use pager or not
+        :param bool interactive: defines, if to use pager
         """
         if interactive:
             tmpfile = tempfile.mkstemp()
@@ -156,15 +147,11 @@ class LMIFormatter(object):
 
 class LMIXmlFormatter(LMIFormatter):
     """
-    XML formatter used for CIMClass.doc() to print pretty verbose help message.
+    XML formatter used for :py:meth:`CIMClass.doc` to print pretty verbose help message.
+
+    :param pywbem.cim_xml.CLASS xml: -- class element
     """
     def __init__(self, xml):
-        """
-        Constructs a object.
-
-        Arguments:
-            xml -- XML element of pywbem.cim_xml.CLASS, retrieved by CIMClass.tocimxml()
-        """
         super(LMIXmlFormatter, self).__init__()
         self._xml = xml
 
@@ -172,13 +159,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a class property.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY node: property element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         children = node.childNodes
         children_len = len(children)
@@ -199,13 +185,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a qualifier.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.QUALIFIER
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.QUALIFIER node: qualifier element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         is_array = isinstance(node.firstChild, pywbem.cim_xml.VALUE_ARRAY)
         val = "[qualifier] %s%s %s" % (node.getAttribute("TYPE"), " []" if is_array else "",
@@ -222,13 +207,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a class property.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERETY
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY node: property element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         val = "[property] %s %s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"))
         LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
@@ -238,13 +222,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a class property array.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY_ARRAY
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY_ARRAY node: property array element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         val = "[property array] %s [] %s" % (node.getAttribute("TYPE"), node.getAttribute("TYPE"))
         LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
@@ -254,13 +237,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a class property reference.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY_REFERENCE
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY_REFERENCE node: property reference element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         val = "[property ref] %s %s" % (node.getAttribute("REFERENCECLASS"), node.getAttribute("NAME"))
         LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
@@ -270,13 +252,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a instance property.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY node: property element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         val = node.firstChild
         prop = "[property] %s %s%s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"),
@@ -287,13 +268,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a instance property array.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY_ARRAY
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY_ARRAY node: property array element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         prop = "[property array] %s [] %s" % (node.getAttribute("TYPE"), node.getAttribute("NAME"))
         val = node.firstChild
@@ -305,13 +285,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a instance property reference.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.PROPERTY_REFERENCE
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.PROPERTY_REFERENCE node: property reference element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         val = "[property ref] %s %s" % (node.getAttribute("REFERENCECLASS"), node.getAttribute("NAME"))
         LMITextFormatter(val).format(indent, sub_indent+4, width, f, separator=True)
@@ -319,13 +298,13 @@ class LMIXmlFormatter(LMIFormatter):
     def __format_method(self, node, indent=0, sub_indent=0, width=80, f=sys.stdout):
         """
         Formats a class method.
-        Arguments:
-            node -- XML element of pywbem.cim_xml.METHOD
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+
+        :param pywbem.cim_xml.METHOD node: method element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
         """
         qualifiers = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.QUALIFIER)]
         parameters = [x for x in node.childNodes if isinstance(x, pywbem.cim_xml.PARAMETER)]
@@ -354,13 +333,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a class.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.CLASS
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.CLASS node: class element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
         """
         LMITextFormatter("Class: %s" % node.getAttribute("NAME")).format(indent, sub_indent, width, f)
         LMITextFormatter("SuperClass: %s" % node.getAttribute("SUPERCLASS")).format(indent+4, sub_indent, width, f)
@@ -380,13 +358,12 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats an instance.
 
-        Arguments:
-            node -- XML element of pywbem.cim_xml.VALUE_NAMEDINSTANCE
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param pywbem.cim_xml.VALUE_NAMEDINSTANCE node: instance element
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: -- output stream
         """
         instance = node.firstChild.nextSibling
         LMITextFormatter("Instance of %s" % instance.getAttribute("CLASSNAME")).format(indent, sub_indent, width, f)
@@ -402,34 +379,38 @@ class LMIXmlFormatter(LMIFormatter):
         """
         Formats a XML object and prints it to the output stream.
 
-        Arguments:
-            indent -- number of spaces to indent the text block
-            sub_indent -- number of spaces for the second and other lines
-                of the text block
-            width -- total text block width
-            f -- output stream
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
         """
         if isinstance(self._xml, pywbem.cim_xml.CLASS):
             self.__format_class(self._xml, indent, sub_indent, width, f)
         elif isinstance(self._xml, pywbem.cim_xml.VALUE_NAMEDINSTANCE):
             self.__format_instance(self._xml, indent, sub_indent, width, f)
+        elif isinstance(self._xml, pywbem.cim_xml.METHOD):
+            self.__format_method(self._xml, indent, sub_indent, width, f)
 
 class LMIMofFormatter(LMIFormatter):
     """
-    MOF formatter used for CIMInstance.doc() to print pretty verbose help message.
+    MOF formatter used for :py:meth:`CIMInstance.doc` to print pretty verbose help
+    message.
+
+    :param string mof: MOF representation of a object
     """
     def __init__(self, mof):
-        """
-        Constructs a object.
-
-        Arguments:
-            mof -- string containing a MOF representation of a object
-        """
         super(LMIMofFormatter, self).__init__()
         self._mof = mof
 
     def format(self, indent=0, sub_indent=0, width=80, f=sys.stdout):
         """
         Formats a MOF object and prints it to the output stream.
+
+        :param int indent: number of spaces to indent the text block
+        :param int sub_indent: number of spaces for the second and other lines
+            of the text block
+        :param int width: total text block width
+        :param f: output stream
         """
         f.write(self._mof)
