@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
+# Copyright (C) 2012-2014 Peter Hatina <phatina@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,16 +14,19 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 import sys
-import time
+import logging
 import pywbem
+import signal
 import socket
 import urlparse
 import threading
+import collections
 
 from LMIBaseObject import LMIWrapperBaseObject
 from LMIBaseClient import LMIBaseClient
 from LMIObjectFactory import LMIObjectFactory
 from LMIFormatter import LMIMofFormatter
+from LMIFormatter import LMIXmlFormatter
 from LMIReturnValue import LMIReturnValue
 from LMIConstantValues import LMIConstantValuesParamProp
 from LMIConstantValues import LMIConstantValuesMethodReturnType
@@ -40,7 +43,7 @@ from LMIJob import JOB_FINISH_EARLY
 
 from LMIUtil import LMIPassByRef
 from LMIUtil import lmi_raise_or_dump_exception
-from LMIUtil import lmi_cast_to_cim
+from LMIUtil import lmi_transform_to_cim_param
 from LMIUtil import lmi_transform_to_lmi
 
 from LMIExceptions import LMIIndicationListenerError
@@ -50,9 +53,93 @@ from LMIExceptions import LMISynchroMethodCallFilterError
 from LMIExceptions import LMIUnknownParameterError
 from LMIExceptions import LMIHandlerNamePatternError
 
+logger = logging.getLogger(__name__)
+
+class LMISignalHelper(object):
+    """
+    Helper class, which takes care of signal (de)registration and handling.
+    """
+
+    _instance = None
+
+    def __new__(cls):
+        if cls._instance is None:
+            cls._instance = super(LMISignalHelper, cls).__new__(cls)
+            LMISignalHelper.reset(cls._instance)
+        return cls._instance
+
+    def reset(self):
+        """
+        Resets the single instance into default state.
+        """
+        self._handler_sigint  = None
+        self._handler_sigterm = None
+        self._signal_handled  = False
+        self._instance._callbacks = collections.OrderedDict()
+
+    def signal_attach(self):
+        """
+        Registers *SIGINT* and *SIGTERM* signals to local handler in which, the flags for
+        each signal are modified, if such signal is caught.
+        """
+        self._signal_handled  = False
+        self._handler_sigint  = signal.signal(signal.SIGINT,  LMISignalHelper.__signal_handler)
+        self._handler_sigterm = signal.signal(signal.SIGTERM, LMISignalHelper.__signal_handler)
+
+    def signal_detach(self):
+        """
+        Unregisters *SIGINT* and *SIGTERM* handler and removes all the attached callbacks.
+        """
+        signal.signal(signal.SIGINT,  self._handler_sigint)
+        signal.signal(signal.SIGTERM, self._handler_sigterm)
+
+    def signal_handled(self):
+        """
+        :returns: True, if any of *SIGINT* or *SIGTERM* has been caught; False otherwise
+        """
+        return self._signal_handled
+
+    def callback_attach(self, cb_name, cb):
+        """
+        Registers a callback, which will be called when a *SIGINT* or *SIGTERM* is caught.
+
+        :param string cb_name: callback name
+        :param cb: callable object, which takes zero arguments
+        """
+        self._callbacks[cb_name] = cb
+
+    def callback_detach(self, cb_name):
+        """
+        Removes a callback from the callback dictionary.
+
+        :param string cb_name: callback name
+        """
+        self._callbacks.pop(cb_name)
+
+    @staticmethod
+    def __signal_handler(signo, frame):
+        """
+        Signal handler, which is called, when *SIGINT* and *SIGTERM* are sent to
+        the LMIShell.
+
+        :param int signo: signal number
+        :param frame: -- stack frame
+
+        **NOTE:** see help(signal)
+        """
+        if signo in (signal.SIGINT, signal.SIGTERM):
+            LMISignalHelper._instance._signal_handled = True
+        for cb in LMISignalHelper._instance._callbacks.values():
+            cb()
+
 class LMIMethod(LMIWrapperBaseObject):
     """
-    LMI wrapper class representing CIMMethod.
+    LMI wrapper class representing :py:class:`CIMMethod`.
+
+    :param LMIConnection conn: connection object
+    :param LMIInstance lmi_instance: :py:class:`LMIInstance` object, on which the method
+        call will be issued
+    :param string method: method name
     """
     # 15 seconds sleep timeout for main waiting thread
     _COND_WAIT_TIME = 15
@@ -80,91 +167,94 @@ class LMIMethod(LMIWrapperBaseObject):
     # less than _POLLING_ADAPT_MAX_WAITING_TIME.
     _POLLING_ADAPT_MAX_WAITING_TIME = 128
 
-    def __init__(self, conn, lmi_instance, method, sync_method):
-        """
-        Constructs a LMIMethod object.
-
-        Arguments:
-            conn -- LMIConnection object
-            lmi_instance -- LMIInstance object, on which the method call will be issued
-            method -- string containing a method name
-            sync_method -- bool flag indicating, if we are trying to perform
-                a synchronous method call
-        """
+    def __init__(self, conn, lmi_instance, method):
         super(LMIMethod, self).__init__(conn)
         self._lmi_instance = lmi_instance
+        self._sync_method = False
+        if method.startswith("Sync"):
+            # Store the synchronous flag and trim the method name
+            self._sync_method = True
+            method = method[4:]
         self._method = lmi_instance._lmi_class._cim_class.methods[method]
-        self._sync_method = sync_method
         # Store the constant values as a list. This can consume some time, if computed on demand.
         self._valuemap_parameters_list = [k for (k, v) in self._method.parameters.iteritems() \
             if "ValueMap" in v.qualifiers]
-        # For simplicity, we add return value constans to the same list
+        # For simplicity, we add return value constants to the same list
         if "ValueMap" in self._method.qualifiers:
             self._valuemap_parameters_list.append(self._method.name)
 
     def __return_synchro_method_call(self, job_inst, job_refresh=True):
         """
-        Returns a LMIReturnValue with Job output parameters set.
+        Returns a :py:class:`LMIReturnValue` object with Job output parameters set.
 
-        Arguments:
-            job_inst -- LMIInstance object of the job returned from a synchronous
-                method call
-            job_refresh -- bool flag, which indicates, if the job_inst
-                needs to be refreshed
+        :param LMIInstance job_inst: job returned from a synchronous method call
+        :param bool job_refresh: flag, which indicates, if the ``job_inst``
+            needs to be refreshed
+        :returns: :py:class:`LMIReturnValue` object with ``rval`` set to Job output parameters
         """
         # Adjust return value from the job object
         if job_refresh:
             job_inst.refresh()
-        rparams = pywbem.NocaseDict({
-                k: x.value
-            for k, x in job_inst.JobOutParameters.properties.iteritems()})
-        rval = rparams["__ReturnValue"]
-        del rparams["__ReturnValue"]    # NocaseDict has no pop()
+        rval = None
+        rparams = pywbem.NocaseDict()
+        if not job_inst.JobOutParameters is None:
+            rparams = pywbem.NocaseDict(dict((k, x.value)
+                for k, x in job_inst.JobOutParameters.properties.iteritems()))
+            rval = rparams["__ReturnValue"]
+            del rparams["__ReturnValue"]    # NocaseDict has no pop()
         errorstr = ""
 
         # Is job in exception state? If so, adjust corresponding error string
         # from job.GetError() instance
         if lmi_is_job_exception(job_inst):
-            (refreshed, _, errorstr) = job_inst.refresh()
+            try:
+                (refreshed, _, errorstr) = job_inst.refresh()
+            except pywbem.cim_operations.CIMError, e:
+                logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                    % (self._method.name, str(e)))
+                job_exception.value = e
             if not refreshed:
+                logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                    % (self._method.name, errorstr))
                 raise LMISynchroMethodCallError(errorstr)
             (exc_rval, exc_rparams, exc_errorstr) = job_inst.GetError()
             error_inst = exc_rparams.get("error", None)
             if not error_inst:
+                logger.debug("Sync%s: Can not retrieve Error instance; %s"
+                    % (self._method.name, exc_errorstr))
                 raise LMISynchroMethodCallError("Could not get Job error message")
             errorstr = error_inst.Message
         return LMIReturnValue(rval=rval, rparams=rparams, errorstr=errorstr)
 
     def __handle_synchro_method_call_indication(self, job_inst):
         """
-        Handles a synchronous call for asynchronous methods returning a job object.
-        This method uses static filters installed by each OpenLMI provider, which is
-        capable of using jobs. Returns LMIReturnValue with rval set to 0, if the
-        synchronous method call was successfuly handled and rparams set to output
-        parameters of the job; otherwise an exception is raised.
+        Handles a synchronous call for asynchronous methods returning a job object. This
+        method uses static filters installed by each OpenLMI provider, which is capable of
+        using jobs.
 
-        Raised exceptions:
-            LMIIndicationListenerError
-            LMISynchroMethodCallError
-            LMISynchroMethodCallFilterError
+        :param LMIInstance job_inst: job object returned from a synchronous
+            method call
+        :returns: :py:class:`LMIReturnValue` object with ``rval`` set to 0 and ``rparams``
+            set to job's output parameters
+        :raises: :py:exc:`.LMIIndicationListenerError`,
+            :py:exc:`.LMISynchroMethodCallError`,
+            :py:exc:`.LMISynchroMethodCallFilterError`
 
-        Static filters' names need to be in format "LMI:<job_class_name>:Changed"
+        **NOTE:** Static filters' names need to be in format
+            "``LMI:<job_class_name>:Changed``"
 
-        Arguments:
-            job_inst -- LMIInstance object of the job returned from a synchronous
-                method call
         """
         def handle_job(ind, cond, job_finished, job_exception):
             """
-            LMIListener handler for synchronous method call, which uses indication means
-            of waiting for the job. This function is called, when a job changes its state.
+            :py:class:`LMIListener` handler for synchronous method call, which uses
+            indication means of waiting for the job. This function is called, when a job
+            changes its state.
 
-            Arguments:
-                cond -- threading.Condition object used for thread synchronization
-                job_finished -- LMIPassByRef object used for indication, whether the
-                    job has finished
-                job_exception -- LMIPassByRef object, which contains an exception object,
-                    if any exception was raised
+            :param threading.Condition cond: condition object used for thread synchronization
+            :param LMIPassByRef job_finished: used for synchronization, whether the
+                job has finished
+            :param LMIPassByRef job_exception: contains an exception object, if any
+                exception was raised
             """
             cond.acquire()
             try:
@@ -205,10 +295,10 @@ class LMIMethod(LMIWrapperBaseObject):
         cim_filter = cim_filters[0]
 
         # Create handler object
-        netloc = urlparse.urlparse(self._conn.hostname).netloc
+        netloc = urlparse.urlparse(self._conn.uri).netloc
         if not netloc:
             listener.stop()
-            errorstr = "Can not determine netloc from client's hostname"
+            errorstr = "Can not determine netloc from client's uri"
             raise LMISynchroMethodCallError(errorstr)
         netloc = netloc.split(":")[0]
         # NOTE: This will work only on a local area network. Complicated networks may require
@@ -222,7 +312,7 @@ class LMIMethod(LMIWrapperBaseObject):
             raise LMISynchroMethodCallError(errorstr)
         destination = s.getsockname()[0]
         s.close()
-        # NOTE: For now, we are using unsecure HTTP as a transport protocol.
+        # NOTE: For now, we are using insecure HTTP as a transport protocol.
         # TODO: Switch to HTTPS
         cim_handler_props = {
             "Name" : indication_name,
@@ -255,37 +345,44 @@ class LMIMethod(LMIWrapperBaseObject):
         if lmi_is_job_finished(job_inst):
             job_finished.value = JOB_FINISH_EARLY
 
+        # Register signal callback for SIGINT, SIGTERM with callback,
+        # which awakes waiting thread for immediate return.
+        LMISignalHelper().callback_attach("indication", lambda: LMIMethod.__wake(cond))
+        LMISignalHelper().signal_attach()
+
         # Wait for the job to finish
-        interrupt = False
-        try:
-            wake_cnt = 0
-            cond.acquire()
-            while True:
-                if interrupt or job_finished.value:
+        wake_cnt = 0
+        cond.acquire()
+        while not LMISignalHelper().signal_handled() and \
+                not job_finished.value and \
+                not lmi_is_job_finished(job_inst):
+            cond.wait(LMIMethod._COND_WAIT_TIME)
+            wake_cnt += 1
+            # XXX: threading.Condition.wait() does not inform about timeout or being awaken by
+            # notify call. There is a counting to 4 sleep cycles before we actually check for
+            # job status manually. This number can be increased, so we rely more on indications,
+            # rather then on manual polling.
+            if wake_cnt >= LMIMethod._COND_WAIT_WAKE_CNT and \
+                    not job_finished.value:
+                wake_cnt = 0
+                try:
+                    (refreshed, _, errorstr) = job_inst.refresh()
+                except pywbem.cim_operations.CIMError, e:
+                    logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                        % (self._method.name, str(e)))
+                    job_exception.value = e
                     break
-                cond.wait(LMIMethod._COND_WAIT_TIME)
-                wake_cnt += 1
-                # XXX: threading.Condition.wait() does not inform about timeout or being awaken by
-                # notify call. There is a counting to 4 sleep cycles before we actually check for
-                # job status manually. This number can be increased, so we rely more on indications,
-                # rather then on manual polling.
-                if not job_finished.value and wake_cnt >= LMIMethod._COND_WAIT_WAKE_CNT:
-                    wake_cnt = 0
-                    try:
-                        (refreshed, _, errorstr) = job_inst.refresh()
-                    except pywbem.cim_operations.CIMError, e:
-                        job_exception.value = e
-                        break
-                    if not refreshed:
-                        job_exception.value = LMISynchroMethodCallError(errorstr)
-                        break
-                    if lmi_is_job_finished(job_inst):
-                        listener.stop()
-                        break
-        except KeyboardInterrupt:
-            interrupt = True
-        finally:
-            cond.release()
+                if not refreshed:
+                    logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                        % (self._method.name, errorstr))
+                    job_exception.value = LMISynchroMethodCallError(errorstr)
+                    break
+
+        # Unregister signal handler
+        LMISignalHelper().signal_detach()
+        LMISignalHelper().callback_detach("indication")
+
+        cond.release()
 
         # Cleanup
         listener.stop()
@@ -293,6 +390,13 @@ class LMIMethod(LMIWrapperBaseObject):
         self._conn._client._delete_instance(cim_handler.path)
         if job_exception.value:
             raise job_exception.value
+        if LMISignalHelper().signal_handled() and not job_finished.value:
+            # We got SIGINT or SIGTERM, when waiting for the job, cancelling the job
+            logger.warn("Cancelling a job '%s'" % job_inst.Name)
+            job_inst.RequestStateChange(
+                RequestedState=job_inst.RequestStateChange.RequestedStateValues.Terminate
+            )
+            return LMIReturnValue(rval=None)
 
         # Return the job return values, refresh the job_inst object, if we got notified
         # about the job finish state by indication.
@@ -301,65 +405,109 @@ class LMIMethod(LMIWrapperBaseObject):
     def __handle_synchro_method_call_polling(self, job_inst):
         """
         Handles a synchronous call for asynchronous methods returning a job object.
-        This call uses polling method to wait for the job to finish. Returns
-        LMIReturnValue object with rval set to 0, if the job was handled with no errors
-        and rparams set to job output parameters; otherwise an exception is raised.
+        This call uses polling method to wait for the job to finish.
 
-        Raised exceptions:
-            LMISynchroMethodCallError
-
-        Arguments:
-            job_inst -- LMIInstance of a job
+        :param LMIInstance job_inst: job object
+        :returns: :py:class:`LMIReturnValue` object with ``rval`` set to 0 and ``rparams``
+            set to job's output parameters.
+        :raises: :py:exc:`.LMISynchroMethodCallError`
         """
+
+        # Register signal callback for SIGINT, SIGTERM with callback,
+        # which awakes waiting thread for immediate return.
+        LMISignalHelper().callback_attach("polling", lambda: LMIMethod.__wake(cond))
+        LMISignalHelper().signal_attach()
+
+        cond = threading.Condition()
+        cond.acquire()
+
+        job_exception = None
+
         try:
             sleep_time = 1
-            while not lmi_is_job_finished(job_inst):
+            while not LMISignalHelper().signal_handled() and \
+                    not lmi_is_job_finished(job_inst):
                 # Sleep, a bit longer in every iteration
-                time.sleep(sleep_time)
+                cond.wait(sleep_time)
                 if sleep_time < LMIMethod._POLLING_ADAPT_MAX_WAITING_TIME:
                     sleep_time *= 2
                 (refreshed, _, errorstr) = job_inst.refresh()
                 if not refreshed:
-                    raise LMISynchroMethodCallError(errorstr)
+                    logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                        % (self._method.name, errorstr))
+                    job_exception = LMISynchroMethodCallError(errorstr)
+                    break
         except pywbem.cim_operations.CIMError, e:
-            raise LMISynchroMethodCallError(e.message)
+            logger.debug("Sync%s: Job instance can not be refreshed; %s"
+                % (self._method.name, str(e)))
+            job_exception = LMISynchroMethodCallError(e.message)
+        finally:
+            cond.release()
+
+        # Unregister signal handler and callback
+        LMISignalHelper().signal_detach()
+        LMISignalHelper().callback_detach("polling")
+
+        if LMISignalHelper().signal_handled() and not lmi_is_job_finished(job_inst):
+            # We got SIGINT or SIGTERM, when waiting for the job, cancelling the job
+            logger.warn("Cancelling a job '%s'" % job_inst.Name)
+            job_inst.RequestStateChange(
+                RequestedState=job_inst.RequestStateChange.RequestedStateValues.Terminate
+            )
+            return LMIReturnValue(rval=None)
+
+        if not job_exception is None:
+            raise job_exception
 
         # Return the job return values. No need to refresh the job instance, we already
         # have a "fresh" one.
         return self.__return_synchro_method_call(job_inst, False)
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, method_args=None, **kwargs):
         """
         Perform a method call.
 
+        Method arguments are preferably passed by dictionary (parameter : value). Using
+        former means of passing arguments, by keyword arguments, to a method call works
+        too.
+
         If performing a synchronous method call, passing PreferPolling can be used
         to select which method should be used -- either subscribing to an indication
-        or polling method. Returns LMIReturnValue object with rval set to remote method
-        call's return value, rparams set to remote method's call return parameters. If an
-        error occurs, LMIReturnValue's errorstr contains the error string. If the LMIShell
-        has exception's propagation enabled, an appropriate exception is raised.
+        or polling method.
 
-        Raised exceptions:
-            LMIUnknownParameterError
-            LMIMethodCallError
-            LMISynchroMethodCallError
+        :param dictionary method_args: method arguments
+        :param dictionary kwargs: keyword method arguments (``method_args`` preffered)
 
-        Arguments:
-            args -- CIMMethod arguments
-            kwargs -- CIMMethod keyword arguments
+            * **RefreshInstance** (*bool*) flag, which tells the LMIShell, whether
+              the instance should be refreshed after a method call. Default
+              value is False.
+
+        :returns: :py:class:`LMIReturnValue` object with ``rval`` set to method's return
+            value; ``rparams`` set to method's return parameters; ``errorstr`` set
+            method's error string in case of failure
+        :raises: :py:exc:`.LMIUnknownParameterError`,
+            :py:exc:`.LMIMethodCallError`
+            :py:exc:`.LMISynchroMethodCallError`
+
+        **Usage:** :ref:`instances_methods`.
         """
         synchro_method_polling = kwargs.pop("PreferPolling", False)
-        for (param, value) in kwargs.iteritems():
+        refresh_requested = kwargs.pop("RefreshInstance", False)
+        if method_args is None:
+            method_args = {}
+        method_args.update(kwargs)
+        for (param, value) in method_args.iteritems():
             if param in self._method.parameters:
+                # Cast input parameters into acceptable CIM types
                 t = self._method.parameters[param].type
-                kwargs[param] = lmi_cast_to_cim(t, value)
+                method_args[param] = lmi_transform_to_cim_param(t, value)
             else:
                 # NOTE: maybe we could check for pywbem type and not to exit prematurely
                 errorstr = "Unknown parameter '%s' supplied for method '%s'" % (param, self._method.name)
                 lmi_raise_or_dump_exception(LMIUnknownParameterError(errorstr))
                 return LMIReturnValue(rval=-1, errorstr=errorstr)
         (rval, call_rparams, call_errorstr) = self._conn._client._call_method_raw(self._lmi_instance,
-            self._method.name, **kwargs)
+            self._method.name, **method_args)
         rval = lmi_transform_to_lmi(self._conn, rval)
         if call_rparams:
             call_rparams = lmi_transform_to_lmi(self._conn, call_rparams)
@@ -401,7 +549,7 @@ class LMIMethod(LMIWrapperBaseObject):
                         lmi_raise_or_dump_exception(e)
                         return LMIReturnValue(rval=-1, errorstr=e.message)
                 call_rparams = lmi_transform_to_lmi(self._conn, call_rparams)
-        if not self._lmi_instance.refresh():
+        if refresh_requested and not self._lmi_instance.refresh():
             # NOTE: this is wrong! What should we do?
             errorstr = "Could not update an LMI object after a method call"
             lmi_raise_or_dump_exception(LMIMethodCallError(errorstr))
@@ -412,8 +560,7 @@ class LMIMethod(LMIWrapperBaseObject):
         """
         Returns either a class member, or a constant value.
 
-        Arguments:
-            name -- string containing the class member, or the constant value name
+        :param string name: class member, or the constant value name
         """
         if name in self.__dict__:
             return self.__dict__[name]
@@ -425,25 +572,45 @@ class LMIMethod(LMIWrapperBaseObject):
                 return LMIConstantValuesMethodReturnType(self._method)
         raise AttributeError(name)
 
+    @staticmethod
+    def __wake(cond):
+        """
+        Helper function used for manual :py:attr:`threading.Condition` wakeup.
+
+        :param threading.Condition cond: condition object
+        """
+        cond.acquire()
+        cond.notify()
+        cond.release()
+
+    def doc(self):
+        """
+        Prints out pretty verbose message with documentation for the class. If the
+        LMIShell is run in a interactive mode, the output will be redirected to a pager
+        set by environment variable :envvar:`PAGER`. If there is not :envvar:`PAGER` set,
+        less or more will be used as a fall-back.
+        """
+        LMIXmlFormatter(self._method.tocimxml()).fancy_format(self._conn._client.interactive)
+
     @property
     def return_type(self):
         """
-        Property returning a string of the method call's return type.
+        :returns: string of the method call's return type
         """
         return self._method.return_type
 
     def tomof(self):
         """
-        Prints out a message with MOF representation of CIMMethod. If the LMIShell
-        is run in a interactive mode, the output will be redirected to a pager set by
-        environment variable PAGER. If there is not PAGER set, less or more will be used
-        as a fallback.
+        Prints out a message with MOF representation of :py:class:`CIMMethod`. If the
+        LMIShell is run in a interactive mode, the output will be redirected to a pager
+        set by environment variable :envvar:`PAGER`. If there is not :envvar:`PAGER` set,
+        less or more will be used as a fall-back.
         """
         LMIMofFormatter(self._method.tomof()).fancy_format(self._conn._client.interactive)
 
     def valuemap_parameters(self):
         """
-        Returns a list of strings of the constant names.
+        :returns: list of strings of the constant names
         """
         return self._valuemap_parameters_list
 
@@ -456,13 +623,20 @@ class LMIMethod(LMIWrapperBaseObject):
 
     def parameters(self):
         """
-        Returns a list of strings of CIMMethod's parameters.
+        :returns: list of strings of :py:class:`CIMMethod`'s parameters
         """
         return self._method.parameters
 
     def print_parameters(self):
         """
-        Prints out CIMMethod's parameters.
+        Prints out :py:class:`CIMMethod`'s parameters.
         """
         for (param, value) in self._method.parameters.iteritems():
             sys.stdout.write("%s %s%s\n" % (value.type, param, "[]" if value.is_array else ""))
+
+    @property
+    def wrapped_object(self):
+        """
+        :returns: wrapped :py:class:`CIMmethod` object
+        """
+        return self._method
