@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
+# Copyright (C) 2012-2014 Peter Hatina <phatina@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -86,12 +86,12 @@ class LMIShellClient(LMIBaseClient):
             :exc:`pywbem.AuthError`
         """
         if self._cache.active:
-            class_list = self._cache.get_classes()
+            class_list = self._cache.get_classes(namespace)
             if class_list is None:
                 (class_list, _, errorstr) = LMIBaseClient._get_class_names(self, namespace, **kwargs)
+                self._cache.set_classes(class_list, namespace)
                 if not class_list:
                     return LMIReturnValue(rval=class_list, errorstr=errorstr)
-                self._cache.set_classes(class_list)
             return LMIReturnValue(rval=class_list)
         return LMIBaseClient._get_class_names(self, namespace, **kwargs)
 
@@ -102,6 +102,8 @@ class LMIShellClient(LMIBaseClient):
         :param string class_name: class name
         :param string namespace: namespace name, from which the class should be retrieved;
             if None, default namespace will be used (**NOTE:** see :mod:`pywbem`)
+        :param bool full_fetch: if specified, it overrides IncludeQualifiers
+            and IncludeClassOrigin parameters
         :param bool LocalOnly:indicates, if only local members should be present in the
             returned :class:`CIMClass`; any CIM elements (properties, methods, and
             qualifiers), except those added or overridden in the class as specified in the
@@ -126,15 +128,25 @@ class LMIShellClient(LMIBaseClient):
         :raises: :exc:`pywbem.CIMError`,
             :exc:`pywbem.AuthError`
         """
+        # Override IncludeQualifiers and IncludeClassOrigin, if full_fetch
+        # parameter was specified.
+        full_fetch = kwargs.pop("full_fetch", None)
+        if not full_fetch is None:
+            kwargs["IncludeQualifiers"] = full_fetch
+            kwargs["IncludeClassOrigin"] = full_fetch
+
         if self._cache.active:
-            klass = self._cache.get_class(class_name)
-            if klass is None:
-                (klass, _, errorstr) = LMIBaseClient._get_class(self, class_name, namespace, **kwargs)
-                if not klass:
-                    return LMIReturnValue(rval=klass, errorstr=errorstr)
-                self._cache.add_class(klass)
-                self._cache.add_superclass(klass.classname, klass.superclass, namespace)
-            return LMIReturnValue(rval=klass)
+            cls_cache_entry = self._cache.get_class(class_name, namespace)
+            if cls_cache_entry is None or (not cls_cache_entry.full_fetch and full_fetch):
+                (cls, _, errorstr) = LMIBaseClient._get_class(
+                    self, class_name, namespace, **kwargs)
+                if cls is None:
+                    return LMIReturnValue(rval=cls, errorstr=errorstr)
+                self._cache.add_class(cls, namespace, full_fetch)
+                self._cache.add_superclass(cls.classname, cls.superclass, namespace)
+            else:
+                cls = cls_cache_entry.cim_class
+            return LMIReturnValue(rval=cls)
         return LMIBaseClient._get_class(self, class_name, namespace, **kwargs)
 
     def _get_superclass(self, classname, namespace=None):
