@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013 Peter Hatina <phatina@redhat.com>
+# Copyright (C) 2012-2014 Peter Hatina <phatina@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ class LMIShellOptionsHelpWithVersionFormatter(argparse.HelpFormatter):
         LMIShell version to the usage message.
         """
         return argparse.HelpFormatter._format_usage(self, usage, actions, groups,
-            "OpenLMI Shell v%s\n\n" % __version__)
+            "LMIShell v%s\n\n" % __version__)
 
 class LMIShellOptionParser(argparse.ArgumentParser):
     """
@@ -76,6 +76,9 @@ class LMIShellOptions(object):
         parser.add_argument("-n", "--noverify", dest="verifycert",
             action="store_false", default=True,
             help="do not verify CIMOM SSL certificate")
+        parser.add_argument("--cwd-first-in-path", dest="cwd_first_in_path",
+            action="store_true", default=False,
+            help="prepend CWD in sys.path instead of appending it")
 
         # Split CLI arguments into LMIShell ones and script ones. LMIShell arguments
         # are those, which are before the first positional argument. Other CLI arguments
@@ -94,6 +97,7 @@ class LMIShellOptions(object):
         # Store CLI options
         self._interact = options.interact
         self._verifycert = options.verifycert
+        self._cwd_first_in_path = options.cwd_first_in_path
         self._log = LMIShellOptions._LOG_DEFAULT
         if options.verbose:
             self._log = LMIShellOptions._LOG_VERBOSE
@@ -163,3 +167,10 @@ class LMIShellOptions(object):
         :rtype: bool
         """
         return self._verifycert
+
+    @property
+    def cwd_first_in_path(self):
+        """
+        :returns: True, if CWD should be prepended in sys.path; False if appended
+        """
+        return self._cwd_first_in_path
